@@ -2,13 +2,17 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
-# 
+# Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+#
 # This file is part of SIP.
-# 
+#
 # This copy of SIP is licensed for use under the terms of the SIP License
 # Agreement.  See the file LICENSE for more details.
-# 
+#
+# This copy of SIP may also used under the terms of the GNU General Public
+# License v2 or v3 as published by the Free Software Foundation which can be
+# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+#
 # SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
@@ -197,7 +201,8 @@ class Makefile:
     """
     def __init__(self, configuration, console=0, qt=0, opengl=0, python=0,
                  threaded=0, warnings=1, debug=0, dir=None,
-                 makefile="Makefile", installs=None, universal=''):
+                 makefile="Makefile", installs=None, universal=None,
+                 arch=None, deployment_target=None):
         """Initialise an instance of the target.  All the macros are left
         unchanged allowing scripts to manipulate them at will.
 
@@ -220,7 +225,11 @@ class Makefile:
         destination.  If the source is another list then it is a set of source
         files and the destination is a directory.
         universal is the name of the SDK if the target is a MacOS/X universal
-        binary.
+        binary.  If it is None then the value is taken from the configuration.
+        arch is the space separated MacOS/X architectures to build.  If it is
+        None then it is taken from the configuration.
+        deployment_target MacOS/X deployment target.  If it is None then it is
+        taken from the configuration.
         """
         if qt:
             if not hasattr(configuration, "qt_version"):
@@ -238,9 +247,6 @@ class Makefile:
         else:
             self._threaded = threaded
 
-        if sys.platform != "darwin":
-            universal = ''
-
         self.config = configuration
         self.console = console
         self._qt = qt
@@ -248,10 +254,33 @@ class Makefile:
         self._python = python
         self._warnings = warnings
         self._debug = debug
-        self._dir = dir
         self._makefile = makefile
         self._installs = installs
-        self._universal = universal
+        self._infix = ""
+
+        # Make sure the destination directory is an absolute path.
+        if dir:
+            self.dir = os.path.abspath(dir)
+        else:
+            self.dir = os.path.curdir
+
+        # Assume we are building in the source tree.
+        self._src_dir = self.dir
+
+        if universal is None:
+            self._universal = configuration.universal
+        else:
+            self._universal = universal
+
+        if arch is None:
+            self._arch = configuration.arch
+        else:
+            self._arch = arch
+
+        if deployment_target is None:
+            self._deployment_target = configuration.deployment_target
+        else:
+            self._deployment_target = deployment_target
 
         self._finalised = 0
 
@@ -356,14 +385,23 @@ class Makefile:
         libdir.extend(self.extra_lib_dirs)
         libdir.extend(self.optional_list("LIBDIR"))
 
-        # Handle MacOS/X universal binaries.
-        if self._universal:
-            unicflags = ('-arch ppc -arch i386 -isysroot %s' % self._universal).split()
-            unilflags = ('-arch ppc -arch i386 -Wl,-syslibroot,%s' % self._universal).split()
+        # Handle MacOS/X specific configuration.
+        if sys.platform == 'darwin':
+            mac_cflags = []
+            mac_lflags = []
 
-            cflags.lextend(unicflags)
-            cxxflags.lextend(unicflags)
-            lflags.lextend(unilflags)
+            for a in self._arch.split():
+                aflag = '-arch ' + a
+                mac_cflags.append(aflag)
+                mac_lflags.append(aflag)
+
+            if self._universal:
+                mac_cflags.append('-isysroot %s' % self._universal)
+                mac_lflags.append('-Wl,-syslibroot,%s' % self._universal)
+
+            cflags.lextend(mac_cflags)
+            cxxflags.lextend(mac_cflags)
+            lflags.lextend(mac_lflags)
 
         # Don't use a unique list as libraries may need to be searched more
         # than once.  Also MacOS/X uses the form "-framework lib" so we don't
@@ -419,7 +457,7 @@ class Makefile:
 
                 libs.append(self.platform_lib(py_lib))
 
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             if win_exceptions:
                 cflags_exceptions = "CFLAGS_EXCEPTIONS_ON"
                 cxxflags_exceptions = "CXXFLAGS_EXCEPTIONS_ON"
@@ -473,7 +511,7 @@ class Makefile:
             cxxflags_debug = "CXXFLAGS_RELEASE"
             lflags_debug = "LFLAGS_RELEASE"
 
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             if self._threaded:
                 cflags.extend(self.optional_list(cflags_mt))
                 cxxflags.extend(self.optional_list(cxxflags_mt))
@@ -502,6 +540,14 @@ class Makefile:
             lflags.extend(self.optional_list("LFLAGS_THREAD"))
 
         if self._qt:
+            # Get the name of the mkspecs directory.
+            try:
+                specd_base = self.config.qt_data_dir
+            except AttributeError:
+                specd_base = self.config.qt_dir
+
+            mkspecs = os.path.join(specd_base, "mkspecs")
+
             if self.generator != "UNIX" and win_shared:
                 defines.append("QT_DLL")
 
@@ -513,8 +559,12 @@ class Makefile:
                     # Note that qmake doesn't define anything for QtHelp.
                     if mod == "QtCore":
                         defines.append("QT_CORE_LIB")
+                    elif mod == "QtDeclarative":
+                        defines.append("QT_DECLARATIVE_LIB")
                     elif mod == "QtGui":
                         defines.append("QT_GUI_LIB")
+                    elif mod == "QtMultimedia":
+                        defines.append("QT_MULTIMEDIA_LIB")
                     elif mod == "QtNetwork":
                         defines.append("QT_NETWORK_LIB")
                     elif mod == "QtOpenGL":
@@ -544,33 +594,40 @@ class Makefile:
             rpaths.extend(libdir_qt)
 
             if self.config.qt_version >= 0x040000:
+                # Try and read QT_LIBINFIX from qconfig.pri.
+                qconfig = os.path.join(mkspecs, "qconfig.pri")
+                self._infix = self._extract_value(qconfig, "QT_LIBINFIX")
+
                 # For Windows: the macros that define the dependencies on
                 # Windows libraries.
                 wdepmap = {
                     "QtCore":       "LIBS_CORE",
                     "QtGui":        "LIBS_GUI",
                     "QtNetwork":    "LIBS_NETWORK",
-                    "QtOpenGL":     "LIBS_OPENGL"
+                    "QtOpenGL":     "LIBS_OPENGL",
+                    "QtWebKit":     "LIBS_WEBKIT"
                 }
 
                 # For Windows: the dependencies between Qt libraries.
                 qdepmap = {
-                    "QtAssistant":      ("QtCore", "QtGui", "QtNetwork"),
+                    "QtAssistant":      ("QtNetwork", "QtGui", "QtCore"),
+                    "QtDeclarative":    ("QtNetwork", "QtGui", "QtCore"),
                     "QtGui":            ("QtCore", ),
-                    "QtHelp":           ("QtCore", "QtGui", "QtSql"),
+                    "QtHelp":           ("QtSql", "QtGui", "QtCore"),
+                    "QtMultimedia":     ("QtGui", "QtCore"),
                     "QtNetwork":        ("QtCore", ),
-                    "QtOpenGL":         ("QtCore", "QtGui"),
+                    "QtOpenGL":         ("QtGui", "QtCore"),
                     "QtScript":         ("QtCore", ),
-                    "QtScriptTools":    ("QtCore", "QtGui", "QtScript"),
+                    "QtScriptTools":    ("QtScript", "QtGui", "QtCore"),
                     "QtSql":            ("QtCore", ),
-                    "QtSvg":            ("QtCore", "QtGui", "QtXml"),
-                    "QtTest":           ("QtCore", "QtGui"),
-                    "QtWebKit":         ("QtCore", "QtGui", "QtNetwork"),
+                    "QtSvg":            ("QtXml", "QtGui", "QtCore"),
+                    "QtTest":           ("QtGui", "QtCore"),
+                    "QtWebKit":         ("QtNetwork", "QtGui", "QtCore"),
                     "QtXml":            ("QtCore", ),
-                    "QtXmlPatterns":    ("QtCore", "QtNetwork"),
-                    "phonon":           ("QtCore", "QtGui"),
-                    "QtDesigner":       ("QtCore", "QtGui"),
-                    "QAxContainer":     ("QtCore", "QtGui")
+                    "QtXmlPatterns":    ("QtNetwork", "QtCore"),
+                    "phonon":           ("QtGui", "QtCore"),
+                    "QtDesigner":       ("QtGui", "QtCore"),
+                    "QAxContainer":     ("QtGui", "QtCore")
                 }
 
                 # The QtSql .prl file doesn't include QtGui as a dependency (at
@@ -622,7 +679,7 @@ class Makefile:
                 # Windows needs the version number appended if Qt is a DLL.
                 qt_lib = self.config.qt_lib
 
-                if self.generator in ("MSVC", "MSVC.NET", "BMAKE") and win_shared:
+                if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE") and win_shared:
                     qt_lib = qt_lib + version_to_string(self.config.qt_version).replace(".", "")
 
                     if self.config.qt_edition == "non-commercial":
@@ -632,15 +689,10 @@ class Makefile:
                 libs.extend(self._dependent_libs(self.config.qt_lib))
 
             # Handle header directories.
-            try:
-                specd_base = self.config.qt_data_dir
-            except AttributeError:
-                specd_base = self.config.qt_dir
-
-            specd = os.path.join(specd_base, "mkspecs", "default")
+            specd = os.path.join(mkspecs, "default")
 
             if not os.access(specd, os.F_OK):
-                specd = os.path.join(specd_base, "mkspecs", self.config.platform)
+                specd = os.path.join(mkspecs, self.config.platform)
 
             incdir.append(specd)
 
@@ -669,9 +721,10 @@ class Makefile:
             libs.extend(self.optional_list("LIBS_OPENGL"))
 
         if self._qt or self._opengl:
-            incdir.extend(self.optional_list("INCDIR_X11"))
-            libdir.extend(self.optional_list("LIBDIR_X11"))
-            libs.extend(self.optional_list("LIBS_X11"))
+            if self.config.qt_version < 0x040000 or self._opengl or "QtGui" in self._qt:
+                incdir.extend(self.optional_list("INCDIR_X11"))
+                libdir.extend(self.optional_list("LIBDIR_X11"))
+                libs.extend(self.optional_list("LIBS_X11"))
 
         if self._threaded:
             libs.extend(self.optional_list("LIBS_THREAD"))
@@ -725,6 +778,8 @@ class Makefile:
         else:
             lib = mname
 
+        lib += self._infix
+
         if self._debug:
             if sys.platform == "win32":
                 lib = lib + "d"
@@ -732,10 +787,11 @@ class Makefile:
                 lib = lib + "_debug"
 
         if sys.platform == "win32" and "shared" in self.config.qt_winconfig.split():
-            if (mname in ("QtCore", "QtDesigner", "QtGui", "QtHelp",
-                          "QtNetwork", "QtOpenGL", "QtScript", "QtScriptTools",
-                          "QtSql", "QtSvg", "QtTest", "QtWebKit", "QtXml",
-                          "QtXmlPatterns", "phonon") or
+            if (mname in ("QtCore", "QtDeclarative", "QtDesigner", "QtGui",
+                          "QtHelp", "QtMultimedia", "QtNetwork", "QtOpenGL",
+                          "QtScript", "QtScriptTools", "QtSql", "QtSvg",
+                          "QtTest", "QtWebKit", "QtXml", "QtXmlPatterns",
+                          "phonon") or
                 (self.config.qt_version >= 0x040200 and mname == "QtAssistant")):
                 lib = lib + "4"
 
@@ -781,7 +837,11 @@ class Makefile:
         flags = []
         prefix = self.optional_string("RPATH")
 
-        if prefix:
+        if prefix == "":
+            # This was renamed in Qt v4.7.
+            prefix = self.optional_string("LFLAGS_RPATH")
+
+        if prefix != "":
             for r in rpaths:
                 flags.append(_quote(prefix + r))
 
@@ -793,7 +853,7 @@ class Makefile:
         clib is the library name in cannonical form.
         framework is set of the library is implemented as a MacOS framework.
         """
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             plib = clib + ".lib"
         elif sys.platform == "darwin" and framework:
             plib = "-framework " + clib
@@ -809,36 +869,43 @@ class Makefile:
         clib is the library name in cannonical form.
         framework is set of the library is implemented as a MacOS framework.
         """
-        prl_libs = []
-
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".prl")
         elif sys.platform == "darwin" and framework:
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".framework", clib + ".prl")
         else:
             prl_name = os.path.join(self.config.qt_lib_dir, "lib" + clib + ".prl")
 
-        if os.access(prl_name, os.F_OK):
+        return self._extract_value(prl_name, "QMAKE_PRL_LIBS").split()
+
+    def _extract_value(self, fname, vname):
+        """Return the stripped value from a name=value line in a file.
+
+        fname is the name of the file.
+        vname is the name of the value.
+        """
+        value = ""
+
+        if os.access(fname, os.F_OK):
             try:
-                f = open(prl_name, "r")
+                f = open(fname, "r")
             except IOError:
-                error("Unable to open \"%s\"" % prl_name)
+                error("Unable to open \"%s\"" % fname)
 
             line = f.readline()
             while line:
                 line = line.strip()
                 if line and line[0] != "#":
                     eq = line.find("=")
-                    if eq > 0 and line[:eq].strip() == "QMAKE_PRL_LIBS":
-                        prl_libs = line[eq + 1:].split()
+                    if eq > 0 and line[:eq].strip() == vname:
+                        value = line[eq + 1:].strip()
                         break
 
                 line = f.readline()
 
             f.close()
 
-        return prl_libs
-
+        return value
 
     def parse_build_file(self, filename):
         """
@@ -851,10 +918,12 @@ class Makefile:
             bfname = "dictionary"
             bdict = filename
         else:
-            if self._dir:
-                bfname = os.path.join(self._dir, filename)
-            else:
+            if os.path.isabs(filename):
+                # We appear to be building out of the source tree.
+                self._src_dir = os.path.dirname(filename)
                 bfname = filename
+            else:
+                bfname = os.path.join(self.dir, filename)
 
             bdict = {}
 
@@ -897,7 +966,7 @@ class Makefile:
                 bdict[i] = ""
 
         # Generate the list of objects.
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             ext = ".obj"
         else:
             ext = ".o"
@@ -945,10 +1014,13 @@ class Makefile:
         """
         self.ready()
 
-        if self._dir:
-            mfname = os.path.join(self._dir, self._makefile)
-        else:
-            mfname = self._makefile
+        # Make sure the destination directory exists.
+        try:
+            os.makedirs(self.dir)
+        except:
+            pass
+
+        mfname = os.path.join(self.dir, self._makefile)
 
         try:
             mfile = open(mfname, "w")
@@ -975,6 +1047,9 @@ class Makefile:
 
         mfile is the file object.
         """
+        if self._deployment_target:
+            mfile.write("export MACOSX_DEPLOYMENT_TARGET = %s\n" % self._deployment_target)
+
         mfile.write("CC = %s\n" % self.required_string("CC"))
         mfile.write("CXX = %s\n" % self.required_string("CXX"))
         mfile.write("LINK = %s\n" % self.required_string("LINK"))
@@ -992,7 +1067,7 @@ class Makefile:
 
         libs = []
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             libdir_prefix = "/LIBPATH:"
         else:
             libdir_prefix = "-L"
@@ -1018,6 +1093,9 @@ class Makefile:
         if self._qt:
             mfile.write("MOC = %s\n" % _quote(self.required_string("MOC")))
 
+        if self._src_dir != self.dir:
+            mfile.write("VPATH = %s\n\n" % self._src_dir)
+
         # These probably don't matter.
         if self.generator == "MINGW":
             mfile.write(".SUFFIXES: .cpp .cxx .cc .C .c\n\n")
@@ -1026,7 +1104,7 @@ class Makefile:
         else:
             mfile.write(".SUFFIXES: .c .cpp .cc .cxx .C\n\n")
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             mfile.write("""
 {.}.cpp{}.obj::
 \t$(CXX) -c $(CXXFLAGS) $(CPPFLAGS) -Fo @<<
@@ -1280,7 +1358,8 @@ class ModuleMakefile(Makefile):
     def __init__(self, configuration, build_file, install_dir=None, static=0,
                  console=0, qt=0, opengl=0, threaded=0, warnings=1, debug=0,
                  dir=None, makefile="Makefile", installs=None, strip=1,
-                 export_all=0, universal=''):
+                 export_all=0, universal=None, arch=None,
+                 deployment_target=None):
         """Initialise an instance of a module Makefile.
 
         build_file is the file containing the target specific information.  If
@@ -1294,11 +1373,10 @@ class ModuleMakefile(Makefile):
         increases the size of the module and slows down module load times but
         may avoid problems with modules that use exceptions.  The default is 0.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs, universal)
+        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs, universal, arch, deployment_target)
 
         self._build = self.parse_build_file(build_file)
         self._install_dir = install_dir
-        self._dir = dir
         self.static = static
 
         self._manifest = ("embed_manifest_dll" in self.optional_list("CONFIG"))
@@ -1370,7 +1448,9 @@ class ModuleMakefile(Makefile):
         self.LFLAGS.extend(self.optional_list(lflags_console))
 
         if sys.platform == "darwin":
-            dl = sys.exec_prefix.split(os.sep)
+            # 'real_prefix' exists if virtualenv is being used.
+            dl = getattr(sys, 'real_prefix', sys.exec_prefix).split(os.sep)
+
             if "Python.framework" not in dl:
                 error("SIP requires Python to be built as a framework")
 
@@ -1467,7 +1547,7 @@ class ModuleMakefile(Makefile):
         mfile.write("\n")
 
         if self.static:
-            if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+            if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
                 mfile.write("LIB = %s\n" % self.required_string("LIB"))
             elif self.generator == "MINGW":
                 mfile.write("AR = %s\n" % self.required_string("LIB"))
@@ -1501,7 +1581,7 @@ class ModuleMakefile(Makefile):
 
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             if self.static:
                 mfile.write("\t$(LIB) /OUT:$(TARGET) @<<\n")
                 mfile.write("\t  $(OFILES)\n")
@@ -1528,10 +1608,7 @@ class ModuleMakefile(Makefile):
                 mfile.write("|\n")
 
                 # Create the .def file that renames the entry point.
-                defname = self._target + ".def"
-
-                if self._dir:
-                    defname = os.path.join(self._dir, defname)
+                defname = os.path.join(self.dir, self._target + ".def")
 
                 try:
                     dfile = open(defname, "w")
@@ -1595,9 +1672,33 @@ class ModuleMakefile(Makefile):
 class SIPModuleMakefile(ModuleMakefile):
     """The class that represents a SIP generated module Makefile.
     """
+    def __init__(self, configuration, build_file, install_dir=None, static=0,
+                 console=0, qt=0, opengl=0, threaded=0, warnings=1, debug=0,
+                 dir=None, makefile="Makefile", installs=None, strip=1,
+                 export_all=0, universal=None, arch=None, prot_is_public=0,
+                 deployment_target=None):
+        """Initialise an instance of a SIP generated module Makefile.
+
+        prot_is_public is set if "protected" is to be redefined as "public".
+        If the platform's C++ ABI allows it this can significantly reduce the
+        size of the generated code.
+
+        For all other arguments see ModuleMakefile.
+        """
+        ModuleMakefile.__init__(self, configuration, build_file, install_dir,
+                static, console, qt, opengl, threaded, warnings, debug, dir,
+                makefile, installs, strip, export_all, universal, arch,
+                deployment_target)
+
+        self._prot_is_public = prot_is_public
+
     def finalise(self):
         """Finalise the macros for a SIP generated module Makefile.
         """
+        if self._prot_is_public:
+            self.DEFINES.append('SIP_PROTECTED_IS_PUBLIC')
+            self.DEFINES.append('protected=public')
+
         self.INCDIR.append(self.config.sip_inc_dir)
 
         ModuleMakefile.finalise(self)
@@ -1609,14 +1710,14 @@ class ProgramMakefile(Makefile):
     def __init__(self, configuration, build_file=None, install_dir=None,
                  console=0, qt=0, opengl=0, python=0, threaded=0, warnings=1,
                  debug=0, dir=None, makefile="Makefile", installs=None,
-                 universal=''):
+                 universal=None, arch=None, deployment_target=None):
         """Initialise an instance of a program Makefile.
 
         build_file is the file containing the target specific information.  If
         it is a dictionary instead then its contents are validated.
         install_dir is the directory the target will be installed in.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs, universal)
+        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs, universal, arch, deployment_target)
 
         self._install_dir = install_dir
 
@@ -1649,6 +1750,9 @@ class ProgramMakefile(Makefile):
 
         build.append(self.required_string("CXX"))
 
+        for a in self._arch.split():
+            build.append('-arch ' + a)
+
         for f in self.optional_list("DEFINES"):
             build.append("-D" + f)
 
@@ -1661,7 +1765,7 @@ class ProgramMakefile(Makefile):
         if self.generator != "BMAKE":
             build.append(source)
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             build.append("-Fe")
             build.append("/link")
             libdir_prefix = "/LIBPATH:"
@@ -1714,7 +1818,7 @@ class ProgramMakefile(Makefile):
     def finalise(self):
         """Finalise the macros for a program Makefile.
         """
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             self.LFLAGS.append("/INCREMENTAL:NO")
 
         if self._manifest:
@@ -1760,15 +1864,18 @@ class ProgramMakefile(Makefile):
         mfile.write("\n$(OFILES): $(HFILES)\n")
 
         for mf in self._build["moc_headers"].split():
-            root, discard = os.path.splitext(mf)
+            root, _ = os.path.splitext(mf)
             cpp = "moc_" + root + ".cpp"
 
+            if self._src_dir != self.dir:
+                mf = os.path.join(self._src_dir, mf)
+
             mfile.write("\n%s: %s\n" % (cpp, mf))
             mfile.write("\t$(MOC) -o %s %s\n" % (cpp, mf))
 
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             mfile.write("\t$(LINK) $(LFLAGS) /OUT:$(TARGET) @<<\n")
             mfile.write("\t  $(OFILES) $(LIBS)\n")
             mfile.write("<<\n")
@@ -2231,8 +2338,10 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
         if line and line[0] != "#":
             assstart = line.find("+")
             if assstart > 0 and line[assstart + 1] == '=':
+                adding = True
                 assend = assstart + 1
             else:
+                adding = False
                 assstart = line.find("=")
                 assend = assstart
 
@@ -2243,6 +2352,11 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 # Remove the escapes for any quotes.
                 rhs = rhs.replace(r'\"', '"').replace(r"\'", "'")
 
+                if adding and rhs != "":
+                    orig_rhs = raw.get(lhs)
+                    if orig_rhs is not None:
+                        rhs = orig_rhs + " " + rhs
+
                 raw[lhs] = rhs
 
         line = f.readline()
@@ -2392,12 +2506,13 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
     return refined
 
 
-def create_wrapper(script, wrapper, gui=0):
+def create_wrapper(script, wrapper, gui=0, use_arch=''):
     """Create a platform dependent executable wrapper around a Python script.
 
     script is the full pathname of the script.
     wrapper is the name of the wrapper file to create.
     gui is non-zero if a GUI enabled version of the interpreter should be used.
+    use_arch is the MacOS/X architecture to invoke python with.
 
     Returns the platform specific name of the wrapper.
     """
@@ -2414,15 +2529,22 @@ def create_wrapper(script, wrapper, gui=0):
 
         wf.write("@\"%s\" \"%s\" %%1 %%2 %%3 %%4 %%5 %%6 %%7 %%8 %%9\n" % (exe, script))
     elif sys.platform == "darwin":
-        # python, pythonw and sys.executable are all different images.  We
-        # would prefer to use the latter (because it includes the version
-        # number) but that would mean being unable to support the "gui"
-        # argument.
+        # The installation of MacOS's python is a mess that changes from
+        # version to version and where sys.executable is useless.
+
         if gui:
             exe = "pythonw"
         else:
             exe = "python"
 
+        version = sys.version_info
+        exe = "%s%d.%d" % (exe, version[0], version[1])
+
+        if use_arch:
+            # Note that this may not work with the "standard" interpreter but
+            # should with the "pythonX.Y" version.
+            exe = "arch -%s %s" % (use_arch, exe)
+
         wf.write("#!/bin/sh\n")
         wf.write("exec %s %s ${1+\"$@\"}\n" % (exe, script))
     else:
